# -*- coding: utf-8 -*-

"""
 (c) 2017 - Copyright Red Hat Inc

 Authors:  The Dream Team
   Pierre-Yves Chibon <pingou@pingoured.fr>

"""
from __future__ import print_function

import collections
import json
import logging
import os

import pagure.config
from pagure.lib import model


if 'PAGURE_CONFIG' not in os.environ \
        and os.path.exists('/etc/pagure/pagure.cfg'):
    os.environ['PAGURE_CONFIG'] = '/etc/pagure/pagure.cfg'


_config = pagure.config.reload_config()
_log = logging.getLogger(__name__)


def main(args):
    """ Creates a JSON blob containing the following structure:
    {
      namespace: {
        package: [people watching issues],
        ...
      },
      ...
    }
    """
    if len(args) != 1:
        print(
            'Please specify the folder where to place the output, and only'
            ' that')
        return 1
    if not os.path.exists(args[0]):
        print('%s does not appear to exist' % args[0])
        return 2
    if not os.path.isdir(args[0]):
        print('%s does not appear to be a directory' % args[0])
        return 3

    session = pagure.lib.create_session(_config['DB_URL'])

    # Start the owner list by the main admin
    query = session.query(
        model.Project.namespace, model.Project.name, model.User.user,
    ).filter(
        model.Project.user_id == model.User.id
    ).filter(
        model.Project.parent_id == None
    ).filter(
        model.Project.is_fork == False
    )

    output = collections.defaultdict(dict)

    for entry in query.all():
        namespace, package, admin = entry
        output[namespace][package] = set([admin])

    # Add the users with commit and admin ACLs

    query = session.query(
        model.Project.namespace, model.Project.name, model.User.user,
    ).filter(
        model.Project.id == model.ProjectUser.project_id
    ).filter(
        model.ProjectUser.user_id == model.User.id
    ).filter(
        model.ProjectUser.access.in_(['commit', 'admin'])
    ).filter(
        model.Project.parent_id == None
    ).filter(
        model.Project.is_fork == False
    )

    for entry in query.all():
        namespace, package, user = entry
        output[namespace][package].add(user)

    # Convert the sets into lists
    final = collections.defaultdict(dict)
    for ns in output:
        for pkg in output[ns]:
            final[ns][pkg] = list(output[ns][pkg])

    with open(os.path.join(args[0], 'pagure_owner_alias.json'), 'w') as stream:
        json.dump(final, stream, indent=4, sort_keys=True)

    session.remove()


if __name__ == '__main__':
    import sys
    sys.exit(main(sys.argv[1:]))
